/*
 *  lightorgan.cpp
 *  polynomeSerial
 *
 *  Created by Colin Mann on 9/18/10.
 *  Copyright 2010 Mannchester Analog. All rights reserved.
 *
 */

#include "lightorgan.h"
#include "SerialDeviceNotifications.h"

static int _LightOrgan_SerialDeviceMessageReceivedCallback(SerialDevice *device, char *data, size_t len, void *userData)
{
	LightOrgan *SELF = (LightOrgan *)userData;
	lorgan_message *message = (lorgan_message *)data;
	//switch (messageGetType(*message)) {
	printf("RECEIVED BYTE0: 0x%X\t\tBYTE1:0x%X\n",message->data0,message->data1);	


	return 0;
}

static void _LightOrgan_SerialDeviceDiscoveredCallback(const char *bsdFilePath, void *userData)
{
    LightOrgan *SELF = (LightOrgan *)userData;
    SELF->getSerialDeviceReader()->addSerialDevice(SELF->getSerialDevice(), 2, _LightOrgan_SerialDeviceMessageReceivedCallback, SELF);
	cout<<"Light Organ Discovered!\n";
}

static void _LightOrgan_SerialDeviceTerminatedCallback(const char *bsdFilePath, void *userData)
{
    LightOrgan *SELF = (LightOrgan *)userData;
	SELF->getSerialDevice()->setUnexpectedDeviceRemovalFlag(true);
    SELF->getSerialDeviceReader()->removeSerialDevice(SELF->getSerialDevice());
	cout<<"Light Organ Disconnected.\n";
}


void LightOrganMidiCallback( double deltatime, std::vector< unsigned char > *message, void *userData )
{
	LightOrgan *SELF = (LightOrgan *)userData;
	char m[1];
	int temp = (message->at(0))&0xF0;
	switch ( temp ) {
		case MIDI_NOTE_OFF:
			m[0] = LB_BULBOFF+((message->at(1))%12);
			SELF->getSerialDevice()->write(m, 1);
			break;
		case MIDI_NOTE_ON:
			if(message->at(2)==0)				//Check for 0 velocity note-on message (same as note-off)
				m[0] = LB_BULBOFF+((message->at(1))%12);
			else
				m[0] = LB_BULBON+((message->at(1))%12);
			SELF->getSerialDevice()->write(m, 1);
			break;
		case MIDI_CC_MESSAGE:
			if((message->at(1))==MIDI_CC_PEDAL)
			{
				if((message->at(2))>64)
					m[0] = LB_PEDAL + 1;
				else
					m[0] = LB_PEDAL;
				SELF->getSerialDevice()->write(m, 1);
			}
			break;
	}
}


LightOrgan::LightOrgan() {

	
}


LightOrgan::~LightOrgan() {
	_LightOrganDeviceReader.stopReading();
	_LightOrganDeviceReader.removeSerialDevice(_LightOrganSerial);
	delete _LightOrganMidiIn;
}


void LightOrgan::start() {
	
	_LightOrganMidiIn = new RtMidiIn();
	_LightOrganMidiIn->openVirtualPort("light bulbs");
	_LightOrganMidiIn->setCallback(LightOrganMidiCallback, this);
	_LightOrganMidiIn->ignoreTypes(false, false, false);
	
	_LightOrganSerial = new SerialDevice("/dev/tty.usbserial-A600e01J", B600);
	_LightOrganSerial->flush();
	
	//Create osc thread
	_LightOrganOscThread = lo_server_thread_new(LIGHTORGAN_LISTEN_PORT, LightOrganOscError);
	_LightOrganOscSender = lo_address_new(NULL, LIGHTORGAN_PORT_SENDER);
	
	lo_server_thread_add_method(_LightOrganOscThread, "/lightorgan/bulb", "ii", LightOrgan_setBulbHandler, this);
	lo_server_thread_add_method(_LightOrganOscThread, NULL,NULL, LightOrgan_TouchHandler, this);
	lo_server_thread_add_method(_LightOrganOscThread, "/lightorgan/mode", "i", LightOrgan_modeHandler, this);
	lo_server_thread_add_method(_LightOrganOscThread, "/lightorgan/clear", "i", LightOrgan_clearHandler, this);
	lo_server_thread_add_method(_LightOrganOscThread, "/lightorgan/home", "i", LightOrgan_setHomeHandler, this);

	SerialDeviceNotificationContext notificationContext = { _LightOrgan_SerialDeviceDiscoveredCallback,
		_LightOrgan_SerialDeviceTerminatedCallback,
		this };
    RegisterForSerialDeviceNotifications(&notificationContext);

	_LightOrganDeviceReader.startReading();
	lo_server_thread_start(_LightOrganOscThread);
	
	cout<<"Light Organ initialized and starting...\n";
}


/*****************OSC MESSAGE HANDLERS***************/
/****************************************************/
void LightOrganOscError(int num, const char *msg, const char *path)
{
    printf("Light Organ server error %d in path %s: %s\n", num, path, msg);
    fflush(stdout);
}

int LightOrgan_TouchHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data)
{
	LightOrgan *SELF = (LightOrgan *)user_data;
	char message[1];
	char buffer[128];
	int i,j;
	for(i=1;i<5;i++)
	{
		for(j=1;j<4;j++)
		{
			sprintf(buffer, "/lightorgan/bulb/%i/%i",i,j);
			//cout<<"The path: "<<buffer<<" match?: "<<lo_pattern_match(path, buffer)<<endl;
			if(lo_pattern_match(path,buffer))
			{
				message[0] = ((int)(argv[0]->f)<<4)+( (4*(j-1))+(i-1) );
				SELF->getSerialDevice()->write(message, 1);
				return 0;
			}
		}
	}
	return 1;
}

int LightOrgan_setBulbHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data)
{
	LightOrgan *SELF = (LightOrgan *)user_data;
	char message[1];
	message[0] = ((LB_BULBOFF + argv[1]->i)<<4)+((argv[0]->i)%12);
	SELF->getSerialDevice()->write(message, 1);
	return 0;
}

int LightOrgan_modeHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data)
{
	LightOrgan *SELF = (LightOrgan *)user_data;
	char message[1];
	message[0] = LB_SETMODE + argv[0]->i;
	SELF->getSerialDevice()->write(message, 1);
	return 0;
}

int LightOrgan_clearHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data)
{
	LightOrgan *SELF = (LightOrgan *)user_data;
	char message[1];
	message[0] = LB_CLEAR + argv[0]->i;
	SELF->getSerialDevice()->write(message, 1);
	return 0;
}

int LightOrgan_setHomeHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data)
{
	LightOrgan *SELF = (LightOrgan *)user_data;
	char message[1];
	message[0] = LB_HOMESET + ((argv[0]->i)%12);
	SELF->getSerialDevice()->write(message, 1);
	return 0;
}

