/*
 *  polynomeOsc.h
 *  polynomeSerial
 *
 *  Created by Colin Mann on 5/26/10.
 *  Copyright 2010 Mannchester Analog. All rights reserved.
 *
 */

#ifndef _POLYNOME_H
#define _POLYNOME_H

#include "lo/lo.h"
#include "types.h"
#include "AsynchronousSerialDeviceReader.h"
#include "SerialDevice.h"
#include <string>
#include "RtMidi.h"


using namespace std;

void polyOscError(int num, const char *msg, const char *path);
int m_LedHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);
int m_ClearHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);
int m_LedColHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);
int m_LedRowHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);
int m_frameHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);
int m_SysPrefixHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);
int m_SysTestHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);
int m_SysReportHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);

int p_ModeHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);
int p_MonoColorHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);
int p_LedHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);
int p_RowHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);
int p_ColHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);
int p_ColorDumpHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);
int p_ArcadeHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);
int p_NixieHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);
int p_UpdateHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);


int intToBCD(int);
unsigned char* myITOA(int);

#define POLYNOME_LISTEN_PORT	"10001"
#define MONOME_LISTEN_PORT		"8080"

#define POLYOSC_PORT_SENDER		"10000"
#define MONOMEOSC_PORT_SENDER	"8000"

#define PMIDI_ANALOG_STARTCC	16
#define PMIDI_ARCADECC			24
#define PMIDI_CC_MESSAGE		176

#define messageGetType(message)                 ((message).data0 & 0xF0) 


using namespace std;

typedef struct {
	uint8 data0;
	uint8 data1;
	
} p_message;

class polynome{

public:

	polynome();
	virtual ~polynome();
	
	void start();
	void setMode(int);
	int getMode();

	lo_server_thread getMonomeServerThread(void) { return _monomeOscThread; }
	lo_server_thread getPolynomeServerThread(void) { return _polyOscThread; }
	
	lo_address getMonomeSender(void) { return _monomeOscSender; }
	lo_address getPolynomeSender(void) { return _polynomeOscSender; }
	
	RtMidiOut * getMidiOut(void) { return _polyMidiOut; }
	
	SerialDevice * getSerialDevice(void) { return _polySerial; }
	
	string getPrefix(void) { return _prefix; }
	void setPrefix(string s) { _prefix = s; }
	

	bool nixieUpdate(void) { return _nixieUpdate; }
	int keypadAnalyze(char);
	
	void clearColorMatrix(void);
	void sendColorMatrix(void);
	
	char colorMatrix[8][8][3];

	
private:
	
	lo_server_thread _polyOscThread;
	lo_server_thread _monomeOscThread;
	
	lo_address _polynomeOscSender;
	lo_address _monomeOscSender;
	
	AsynchronousSerialDeviceReader _polyDeviceReader;
	SerialDevice *_polySerial;
	
	RtMidiOut *_polyMidiOut;
	RtMidiIn *_polyMidiIn;
	
	string	_prefix;
	int		_mode;
	
	string			_nixieDisplay;
	int				_nixieBCD;
	bool			_nixieUpdate;
	
};




#endif