/*
 *  relayBox.cpp
 *  polynomeSerial
 *
 *  Created by Colin Mann on 6/25/10.
 *  Copyright 2010 Mannchester Analog. All rights reserved.
 *
 */

#include <stdio.h>
#include <iostream>
#include <stdint.h>
#include "lo/lo.h"
#include "SerialDevice.h"
#include "relayBox.h"

using namespace std;

static int _Relay_SerialDeviceMessageReceivedCallback(SerialDevice *device, char *data, size_t len, void *userData) {

	relay *SELF = (relay *)userData;
	vector<unsigned char> midimessage;
	r_message *message = (r_message *)data;
	switch (messageGetType(*message)) {
		case SWITCHON:
			midimessage.push_back(RMIDI_CC_MESSAGE);
			midimessage.push_back( ((message->data0)&0x0F)+RMIDI_CC_START);
			midimessage.push_back(127);
			SELF->getMidiOut()->sendMessage( &midimessage );
			 break;
		case SWITCHOFF:
			midimessage.push_back(RMIDI_CC_MESSAGE);
			midimessage.push_back( ((message->data0)&0x0F)+RMIDI_CC_START);
			midimessage.push_back(0);
			SELF->getMidiOut()->sendMessage( &midimessage );
			 break;
		case DEBUG:
			switch (message->data1) {
				case UART_FRAME_ERROR:
					printf("RELAY:A framing error has occured on the UART bus.\n");
					break;
				case UART_OVERRUN_ERROR:
					printf("RELAY:A buffer overrun error has occured, sending data too fast.\n");
					break;
				case UART_BUFFER_OVERFLOW:
					printf("RELAY:A buffer overflow error has occured, sending data too fast.\n");
					break;
				default:
					printf("RELAY:DEBUG:DEFAULT: byte 0x%X\n",message->data1);
			}
			break;
		default:
			printf("RELAY:DEFAULT:Byte 0 is: 0x%X, Byte 1 is: 0x%X\n",(message->data0),message->data1);
			return -1;
			break;
	}
	return 0;
}

void relayMidiCallback( double deltatime, std::vector< unsigned char > *message, void *user_data )
{
	char m[1];
	relay *SELF = (relay *)user_data; 
	
	unsigned int nBytes = message->size();
	for ( unsigned int i=0; i<nBytes; i++ )
		std::cout << "Byte " << i << " = " << (int)message->at(i) << ", ";
	if ( nBytes > 0 )
		std::cout << "stamp = " << deltatime << std::endl;
	
	if ( (message->at(0)==NOTEOFF) || (message->at(2)==0) )
	{
		int relayChannel = (message->at(1))-RMIDI_START;
		if ( (relayChannel>=0) && (relayChannel<8) ) {
			m[0] = RELAYOFF + relayChannel;
			SELF->getSerialDevice()->write(m, 1);
			printf("got here. message is: 0x%X\n",m[0]);
		}
	}
	if ( (message->at(0)==NOTEON) && (message->at(2)>0) ) {
			int relayChannel =(message->at(1))-RMIDI_START;
			if ( (relayChannel>=0) && (relayChannel<8) )
			{
				m[0] = RELAYON + relayChannel;
				SELF->getSerialDevice()->write(m, 1);
			}
	}
	
}

relay::relay() {
	
}

relay::~relay() {
	_relayDeviceReader.stopReading();
	_relayDeviceReader.removeSerialDevice(_relaySerial);
	_relayMidiOut->closePort();
	delete _relayMidiOut;
	delete _relayMidiIn;
}

void relay::start() {
	_relayMidiOut = new RtMidiOut();
	_relayMidiOut->openVirtualPort("relaybox");
	
	_relayMidiIn = new RtMidiIn();
	_relayMidiIn->openVirtualPort("relaybox");
	_relayMidiIn->setCallback(&relayMidiCallback, this);
	_relayMidiIn->ignoreTypes(false, false, false);
	
	_relaySerial = new SerialDevice("/dev/tty.usbserial-0000201D",B38400);
	_relayDeviceReader.addSerialDevice(_relaySerial, 2, _Relay_SerialDeviceMessageReceivedCallback, this);
	_relaySerial->flush();
	
	//Create osc thread
	_relayOscThread = lo_server_thread_new(RELAY_LISTEN_PORT, relayOscError);
	//Create osc message senders
	_relayOscSender = lo_address_new(NULL, RELAY_PORT_SENDER);	//Null address is localmachine
	
	
	//Relay server methods
	lo_server_thread_add_method(_relayOscThread, ("/relay"), "ii", relayChannelHandler, this);
	lo_server_thread_add_method(_relayOscThread, ("/relay"), "i", relayHandler, this);
	
	_relayDeviceReader.startReading();
	lo_server_thread_start(_relayOscThread);
	
	cout<<"Relay Box initialized and starting...\n";
}


/*****************OSC MESSAGE HANDLERS***************/
/****************************************************/
void relayOscError(int num, const char *msg, const char *path)
{
    printf("relay server error %d in path %s: %s\n", num, path, msg);
    fflush(stdout);
}


int relayChannelHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data)
{
	relay *SELF = (relay *)user_data;
	char message[1];
	int channel = argv[0]->i;
	if (channel<8) {
		message[0] = ((argv[1]->i)<<4) + channel;
		SELF->getSerialDevice()->write(message, 1);
	}
	fflush(stdout);
	return 0;
}

int relayHandler(const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data)
{
	relay *SELF = (relay *)user_data;
	char message[2];
	message[0] = RB_SETALL;
	message[1] = argv[0]->i;
	SELF->getSerialDevice()->write(message, 2);
	fflush(stdout);
	return 0;
}



